import datetime
import os


def dict_compare(old_dict, new_dict, nested=None):
    """ Compare two dictionaries
        Only 1 level, ignoring attributes starting with '_'
    """
    key_prefix = nested + '.' if nested else ''
    intersect_keys = old_dict.keys() & new_dict.keys()
    modified = {key_prefix + k: dict(old=old_dict[k], new=new_dict[k], action='mod') for k in intersect_keys
                        if k[0] != '_'
                        and old_dict[k] != new_dict[k]
                        and not (isinstance(old_dict[k], dict) and isinstance(new_dict[k], dict))}

    nested_keys = [k for k in intersect_keys if k[0] != '_' and isinstance(old_dict[k], dict) and isinstance(new_dict[k], dict)]
    for k in nested_keys:
        x = dict_compare(old_dict[k], new_dict[k], key_prefix + k)
        if x:
            modified.update(x)

    added = new_dict.keys() - old_dict.keys()
    modified.update({key_prefix + k: dict(new=new_dict[k], action='add') for k in added if k[0] != '_'})

    deleted = old_dict.keys() - new_dict.keys()
    modified.update({key_prefix + k: dict(old=old_dict[k], action='del') for k in deleted if k[0] != '_'})
    if modified:
        return modified


def running_in_gcf():
    """ Determine if code is running in GCF using X_GOOGLE_FUNCTION_NAME
    """
    return os.getenv('X_GOOGLE_FUNCTION_NAME') is not None


def default_object(o):
    """ Default handler for json.dumps()"""
    if isinstance(o, (datetime.date, datetime.datetime)):
        return o.isoformat()
    raise TypeError("Type %s not serializable" % type(o))


def buid_dict(composed_key, key_value):
    def recursive_buid_dict(keys, value, nest_level=0):
        if nest_level < len(keys):
            return recursive_buid_dict(keys, value={keys[nest_level]: value}, nest_level=nest_level + 1)
        elif nest_level == len(keys):
            return value

    return recursive_buid_dict(keys=list(reversed(composed_key.split('.'))), value=key_value)

from collections import ChainMap

def merge_dicts(dict_a, dict_b):
    new_dict = dict()
    for key in ChainMap(dict_a, dict_b):
        val_a , val_b = dict_a.get(key), dict_b.get(key)
        if isinstance(val_a, dict) and isinstance(val_b, dict):
            new_dict[key] = merge_dicts(val_a, val_b)
        elif val_a is None and val_b is None:
            continue
        elif val_a is None:
            new_dict[key] = val_b
        elif val_b is None or val_a == val_b:
            new_dict[key] = val_a
        else:
            new_dict[key] = [val_a, val_b]
    return new_dict



import json
import logging

from google.cloud import pubsub

from ijr.generic_lib import running_in_gcf, default_object


class PubSubPublisher(object):
    _threshold = 25

    def __init__(self, topic, msg_type='Generic', msg_source='default'):
        self._messages = list()
        self._topic = topic
        self.msg_type = msg_type
        self._msg_source = msg_source
        if running_in_gcf():
            self._client = pubsub.PublisherClient()
        else:
            logging.warning('PubSubPublisher -> Running local; using ./account.json')
            self._client = pubsub.PublisherClient.from_service_account_json('account.json')

    def __enter__(self):
        return self

    def __exit__(self, exc_type, exc_val, exc_tb):
        self._publish_messages()

    @property
    def msg_type(self):
        return self.__msg_type

    @msg_type.setter
    def msg_type(self, prop_value):
        if prop_value is None:
            raise Exception("msg_type can't be None")
        self.__msg_type = prop_value

    def _publish_messages(self):
        if not self._messages:
            return
        msg_dict = dict(_type=self.msg_type,
                        data=self._messages)
        msg = json.dumps(msg_dict, sort_keys=True, default=default_object)
        ret = self._client.publish(self._topic, msg.encode(), source=self._msg_source).result()
        self._messages.clear()
        return ret

    def publish(self, msg):
        self._messages.append(msg)
        if len(self._messages) > self._threshold:
            self._publish_messages()



from setuptools import setup


__version__ = '0.0.6'

setup(
    name='ijr',
    version=__version__,
    packages=['ijr'],
    url='https://github.com/IJsvogel/ijr',
    license='MIT',
    author='Herman Holterman',
    author_email='herman.holterman@gmail.com',
    description='IJsvogel Package',
    install_requires=['pymongo', 'google-cloud-pubsub']
)
